/*
 * init.S --	Startup code for loader.c, which runs in SRAM.
 * 
 * Copyright (C) 1999 Ben Williamson <benw@pobox.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define SRAM_BASE	0x80000000	/* in bootstrap mode */
#define SRAM_OFFSET	0x00014000
#define SRAM_SIZE	0x00001100	/* EP9312 has 4.2kB */

.text
	.global _start
_start:
	mov	sp, #SRAM_BASE
#ifdef SRAM_OFFSET
	add	sp, sp, #SRAM_OFFSET
#endif
	add	sp, sp, #SRAM_SIZE	/* stack in SRAM */
	mov     r0, #0
	ldr	r1, =__bss_start	/* clear bss */
1:	str	r0, [r1], #4
	cmp	r1, sp
	blo	1b

	bl	main			/* see loader.c */
2:	b	2b

	.global flush_v3
flush_v3:
	mov     r0, #0
	mcr     p15, 0, r0, c7, c7, 0	/* flush v3/v4 cache */
	mcr	p15, 0, r0, c5, c0, 0	/* flush v3 TLB */
	mov	pc, lr

	.global flush_v4
flush_v4:
	mov     r0, #0
	mcr     p15, 0, r0, c7, c7, 0	/* flush v3/v4 cache */
	mcr	p15, 0, r0, c8, c7, 0	/* flush v4 TLB */
	mov	pc, lr

	.global async_mode
async_mode:
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #0xc0000000
	mcr	p15, 0, r0, c1, c0, 0
	mov	pc, lr

	.global set_clock1
set_clock1:
	mov	r0, #0x80000000
	orr	r0, r0, #0x930000
	mov	r1, #0x02000000
	orr	r1, r1, #0xa40000
	orr	r1, r1, #0xe300
	orr	r1, r1, #0x9e
	str	r1, [r0, #0x20]
	nop
	nop
	nop
	nop
	nop
	mov	pc, lr

	.global set_clock2
set_clock2:
	mov	r0, #0x80000000
	orr	r0, r0, #0x930000
	mov	r1, #0x03000000
	orr	r1, r1, #0x0d0000
	orr	r1, r1, #0xc300
	orr	r1, r1, #0x17
	str	r1, [r0, #0x24]
	mov	pc, lr

	.global	uartbaud_115200
uartbaud_115200:
	mov	r0, #0x80000000
	orr	r0, r0, #0x930000
	ldr	r1, [r0, #0x04]
	orr	r1, r1, #0x20000000
	str	r1, [r0, #0x04]
	mov	r1, #0x95
	mov	r0, #0x80000000
	orr	r0, r0, #0x8c0000
	mov	r1, #0
	str	r1, [r0, #0x014]
	str	r1, [r0, #0x100]
	str	r1, [r0, #0x004]
	str	r1, [r0, #0x00c]
	mov	r1, #7
	str	r1, [r0, #0x010]
	mov	r1, #0x60
	str	r1, [r0, #0x008]
	str	r1, [r2, #0x008]
	mov	r1, #1
	str	r1, [r0, #0x014]
	str	r1, [r2, #0x014]
	mov	pc, lr
